// Example of use : head.over_family().sum(msl::var::disposable_income)
template<msl::Individual* msl::Individual::*next_ptr>
double
CircConstIterAsForwardConstIterWrapper<next_ptr>::sum(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("sum", get.name_()) };

	return std::accumulate(this->cbegin(),
	                       this->cend(),
	                       double{},
	                       [&](double sum, msl::RefConst ref) { return sum + get.flt_(ref); });
}
template<msl::Individual* msl::Individual::*next_ptr>
double
CircConstIterAsForwardConstIterWrapper<next_ptr>::min(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("sum", get.name_()) };

	return get.flt_(
	  *std::min_element(this->cbegin(), this->cend(), [&](msl::RefConst ref1, msl::RefConst ref2) {
		  double const x1{ get.flt_(ref1) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x1))
			  msl::error_get_nan_interest_variable("min", get.name_(), ref1);
		  double const x2{ get.flt_(ref2) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x2))
			  msl::error_get_nan_interest_variable("min", get.name_(), ref2);
		  return x1 < x2;
	  }));
}
template<msl::Individual* msl::Individual::*next_ptr>
msl::RefConst
CircConstIterAsForwardConstIterWrapper<next_ptr>::ref_min(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("ref_min", get.name_()) };

	return *std::min_element(
	  this->cbegin(), this->cend(), [&](msl::RefConst ref1, msl::RefConst ref2) {
		  double const x1{ get.flt_(ref1) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x1))
			  msl::error_get_nan_interest_variable("ref_min", get.name_(), ref1);
		  double const x2{ get.flt_(ref2) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x2))
			  msl::error_get_nan_interest_variable("ref_min", get.name_(), ref2);
		  return x1 < x2;
	  });
}
template<msl::Individual* msl::Individual::*next_ptr>
double
CircConstIterAsForwardConstIterWrapper<next_ptr>::max(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("max", get.name_()) };

	return get.flt_(
	  *std::max_element(this->cbegin(), this->cend(), [&](msl::RefConst ref1, msl::RefConst ref2) {
		  double const x1{ get.flt_(ref1) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x1))
			  msl::error_get_nan_interest_variable("max", get.name_(), ref1);
		  double const x2{ get.flt_(ref2) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x2))
			  msl::error_get_nan_interest_variable("max", get.name_(), ref2);
		  return x1 < x2;
	  }));
}
template<msl::Individual* msl::Individual::*next_ptr>
msl::RefConst
CircConstIterAsForwardConstIterWrapper<next_ptr>::ref_max(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("ref_max", get.name_()) };

	return *std::max_element(this->begin(), this->end(), [&](msl::RefConst ref1, msl::RefConst ref2) {
		double const x1{ get.flt_(ref1) };
		// Check if the value is not a missing one. Display an error and exit.
		if (std::isnan(x1))
			msl::error_get_nan_interest_variable("ref_max", get.name_(), ref1);
		double const x2{ get.flt_(ref2) };
		// Check if the value is not a missing one. Display an error and exit.
		if (std::isnan(x2))
			msl::error_get_nan_interest_variable("ref_max", get.name_(), ref2);
		return x1 < x2;
	});
}
template<msl::Individual* msl::Individual::*next_ptr>
double
CircConstIterAsForwardConstIterWrapper<next_ptr>::avg(msl::Table const& get) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("avg", get.name_()) };

	std::pair<double, size_t> pair{ std::accumulate(
		this->cbegin(),
		this->cend(),
		std::pair<double, size_t>{},
		[&](std::pair<double, size_t> pair, msl::RefConst ref) {
		  float const x{ get.flt_(ref) };
		  // Check if the value is not a missing one. Display an error and exit.
		  if (std::isnan(x))
			  msl::error_get_nan_interest_variable("avg", get.name_(), ref);
		  return std::pair<double, size_t>{ pair.first + x, ++pair.second };
		}) };
	return pair.second == 0 ? std::numeric_limits<double>::quiet_NaN() : pair.first / pair.second;
}
