namespace var {
namespace internals {
struct Table
{
	// To get a float value, from a CTG, FLG or FLT data member, used e. g. in the ''msl::Population
	// ::sum'' member function. For a FLT data member, throw a error message if the data member is
	// equal to NaN as the error message is more accurate: we can give the name of the variable.
	float (*flt_)(msl::RefConst);
	// To get a uint8_t value, from a CTG data member, used e. g. in the ''msl::Population::sum_by''
	// member function.
	uint8_t (*ctg_)(msl::RefConst);
	// To set a bool, a float data or a uint8_t member, used e. g. in the ''msl::Population::fill'' or
	// in the ''msl::Population::fill_if'' member functions. Only alive individuals are setted.
	void (*set_)(msl::Ref, float);
	// To set a float data  member to missing value, on all individuals.
	void (*mis_)(msl::Ref);
	// To get the variable name, used e. g. in the ''dump'' free function.
	char const* (*name_)();
	// To get the value representation, used e. g. in the ''dump'' free function.
	std::str (*str_)(msl::RefConst);
};
} // namespace internals

#define CTG(prm, format)                                                                           \
	namespace internals {                                                                            \
	inline float flt_##prm(msl::RefConst ref)                                                        \
	{                                                                                                \
		return ref.prm;                                                                                \
	}                                                                                                \
	inline uint8_t ctg_##prm(msl::RefConst ref)                                                      \
	{                                                                                                \
		return ref.prm;                                                                                \
	}                                                                                                \
	inline void set_##prm(msl::Ref ref, float x)                                                     \
	{                                                                                                \
		ref.prm = x;                                                                                   \
	}                                                                                                \
	inline char const* name_##prm()                                                                  \
	{                                                                                                \
		return #prm;                                                                                   \
	}                                                                                                \
	inline std::str str_##prm(msl::RefConst ref)                                                     \
	{                                                                                                \
		return std::to_string(size_t{ ref.prm });                                                      \
	}                                                                                                \
	}                                                                                                \
	msl::var::internals::Table constexpr prm{                                                        \
		msl::var::internals::flt_##prm,  msl::var::internals::ctg_##prm,                               \
		msl::var::internals::set_##prm,  nullptr,                                                      \
		msl::var::internals::name_##prm, msl::var::internals::str_##prm,                               \
	};

#define FLG(prm)                                                                                   \
	namespace internals {                                                                            \
	inline float flt_##prm(msl::RefConst ref)                                                        \
	{                                                                                                \
		return ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm)];                             \
	}                                                                                                \
	inline void set_##prm(msl::Ref ref, float x)                                                     \
	{                                                                                                \
		if (ref.alive())                                                                               \
			ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm)] = x;                              \
	}                                                                                                \
	inline char const* name_##prm()                                                                  \
	{                                                                                                \
		return #prm;                                                                                   \
	}                                                                                                \
	inline std::str str_##prm(msl::RefConst ref)                                                     \
	{                                                                                                \
		return ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm)] ? "Yes" : "No";              \
	}                                                                                                \
	}                                                                                                \
	msl::Table constexpr prm{                                                                        \
		msl::var::internals::flt_##prm,  nullptr,                                                      \
		msl::var::internals::set_##prm,  nullptr,                                                      \
		msl::var::internals::name_##prm, msl::var::internals::str_##prm,                               \
	};

#define FLT(prm)                                                                                   \
	namespace internals {                                                                            \
	inline float flt_##prm(msl::RefConst ref)                                                        \
	{                                                                                                \
		return ref.prm;                                                                                \
	}                                                                                                \
	inline void set_##prm(msl::Ref ref, float x)                                                     \
	{                                                                                                \
		if (ref.alive())                                                                               \
			ref.prm = x;                                                                                 \
	}                                                                                                \
	inline void mis_##prm(msl::Ref ref)                                                              \
	{                                                                                                \
		ref.prm = std::numeric_limits<float>::quiet_NaN();                                             \
	}                                                                                                \
	inline char const* name_##prm()                                                                  \
	{                                                                                                \
		return #prm;                                                                                   \
	}                                                                                                \
	inline std::str str_##prm(msl::RefConst ref)                                                     \
	{                                                                                                \
		return std::isnan(ref.prm) ? "." : std::to_string(ref.prm);                                    \
	}                                                                                                \
	}                                                                                                \
	msl::Table constexpr prm{                                                                        \
		msl::var::internals::flt_##prm,  nullptr,                                                      \
		msl::var::internals::set_##prm,  msl::var::internals::mis_##prm,                               \
		msl::var::internals::name_##prm, msl::var::internals::str_##prm,                               \
	};

#define LNK(prm)                                                                                   \
	namespace internals {                                                                            \
	inline float flt_##prm##_head(msl::RefConst ref)                                                 \
	{                                                                                                \
		return ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm##_head)];                      \
	}                                                                                                \
	inline void set_##prm##_head(msl::Ref ref, float x)                                              \
	{                                                                                                \
		ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm##_head)] = x;                         \
	}                                                                                                \
	inline char const* name_##prm##_head()                                                           \
	{                                                                                                \
		return #prm "_head";                                                                           \
	}                                                                                                \
	inline std::str str_##prm##_head(msl::RefConst ref)                                              \
	{                                                                                                \
		return ref.flags_[static_cast<int>(msl::Individual::FLAGS_::prm##_head)] ? "Yes" : "No";       \
	}                                                                                                \
                                                                                                   \
	inline char const* name_##prm##_next()                                                           \
	{                                                                                                \
		return #prm "_next";                                                                           \
	}                                                                                                \
	inline std::str str_##prm##_next(msl::RefConst ref)                                              \
	{                                                                                                \
		return ref.prm##_next ? std::to_string(ref.prm##_next->id_number) : ".";                       \
	}                                                                                                \
                                                                                                   \
	inline float flt_##prm##_weight(msl::RefConst ref)                                               \
	{                                                                                                \
		return ref.prm##_weight;                                                                       \
	}                                                                                                \
	inline void set_##prm##_weight(msl::Ref ref, float x)                                            \
	{                                                                                                \
		ref.prm##_weight = x;                                                                          \
	}                                                                                                \
	inline void mis_##prm##_weight(msl::Ref ref)                                                     \
	{                                                                                                \
		ref.prm##_weight = std::numeric_limits<float>::quiet_NaN();                                    \
	}                                                                                                \
	inline char const* name_##prm##_weight()                                                         \
	{                                                                                                \
		return #prm "_weight";                                                                         \
	}                                                                                                \
	inline std::str str_##prm##_weight(msl::RefConst ref)                                            \
	{                                                                                                \
		return std::isnan(ref.prm##_weight) ? "." : std::to_string(ref.prm##_weight);                  \
	}                                                                                                \
	}                                                                                                \
	msl::Table constexpr prm##_head{                                                                 \
		msl::var::internals::flt_##prm##_head,  nullptr,                                               \
		msl::var::internals::set_##prm##_head,  nullptr,                                               \
		msl::var::internals::name_##prm##_head, msl::var::internals::str_##prm##_head,                 \
	};                                                                                               \
	msl::Table constexpr prm##_next{                                                                 \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		msl::var::internals::name_##prm##_next,                                                        \
		msl::var::internals::str_##prm##_next,                                                         \
	};                                                                                               \
	msl::Table constexpr prm##_weight{                                                               \
		msl::var::internals::flt_##prm##_weight,  nullptr,                                             \
		msl::var::internals::set_##prm##_weight,  msl::var::internals::mis_##prm##_weight,             \
		msl::var::internals::name_##prm##_weight, msl::var::internals::str_##prm##_weight,             \
	};

#define PTR(prm)                                                                                   \
	namespace internals {                                                                            \
	inline char const* name_##prm()                                                                  \
	{                                                                                                \
		return #prm;                                                                                   \
	}                                                                                                \
	inline std::str str_##prm(msl::RefConst ref)                                                     \
	{                                                                                                \
		return ref.prm ? std::to_string(ref.prm->id_number) : ".";                                     \
	}                                                                                                \
	}                                                                                                \
	msl::Table constexpr prm{                                                                        \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		nullptr,                                                                                       \
		msl::var::internals::name_##prm,                                                               \
		msl::var::internals::str_##prm,                                                                \
	};
FLG(one)
#include "msl-variables.cpp"
#undef PTR
#undef CTG
#undef FLG
#undef FLT
#undef LNK
} // namespace var
