struct Population : public PopBase
{

	// Call the base class constructor.
	Population(size_t n)
	  : PopBase(n)
	{
	}

	void fill(msl::Table const& set, float value, char const* f_name = "fill")
	{
		if (set.set_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		// Only alive individuals are setted by the ''set_'' free function.
		(void)std::for_each(std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) {
			set.set_(ref, value);
		});
	}
	void fill_if(msl::Table const& set,
	             float value,
	             msl::Lambda const& filter,
	             char const* f_name = "fill_if")
	{
		if (set.set_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		// Only alive individuals are setted by the ''set_'' free function.
		(void)std::for_each(std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) {
			if (filter(ref))
				set.set_(ref, value);
		});
	}
	void fill_if(msl::Table const& set,
	             float value,
	             msl::Table const& filter,
	             char const* f_name = "fill_if")
	{
		if (set.set_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		// Only alive individuals are setted by the ''set_'' free function.
		(void)std::for_each(std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) {
			if (filter.flt_(ref))
				set.set_(ref, value);
		});
	}

	void fill_with_missing(msl::Table const& set, char const* f_name = "fill_with_missing")
	{
		if (set.mis_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		// All the individuals are setted by the ''mis_'' free function.
		(void)std::for_each(
		  std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) { set.mis_(ref); });
	}

	void fill_if_with_missing(msl::Table const& set,
	                          msl::Table const& filter,
	                          char const* f_name = "fill_if_with_missing")
	{
		if (set.mis_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		// All the individuals are setted by the ''mis_'' free function.
		(void)std::for_each(std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) {
			if (filter.flt_(ref))
				set.mis_(ref);
		});
	}

	void fill_if_with_missing(msl::Table const& set,
	                          msl::Lambda const& filter,
	                          char const* f_name = "fill_if_with_missing")
	{
		if (set.mis_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, set.name_()) };
		// All the individuals are setted by the ''mis_'' free function.
		(void)std::for_each(std::execution::par_unseq, this->begin(), this->end(), [&](msl::Ref ref) {
			if (filter(ref))
				set.mis_(ref);
		});
	}

	void for_each(msl::Function const&);

	void for_each_if(msl::Function const&, msl::Lambda const&);

	void for_each_if(msl::Function const&, msl::Table const&);

	size_t count_if(msl::Table const&) const;

	size_t count_if(msl::Lambda const&) const;

	double weighted_count_if(msl::Lambda const&, msl::Table const&) const;

	// Return the reference of a free slot to represent an individual. The returned individual is in a
	// valid state.
	msl::Ref birth(msl::Ref parent,
	               int birthday = msl::days_from_civil(0, 1, 1),
	               size_t id_number = 0);

	// Return the reference of the ''id_number” individual.
	msl::RefConst find(size_t id_number) const;
	msl::Ref find(size_t id_number);

	size_t align(size_t, msl::Lambda const&, msl::Function const&, bool = false);
	size_t align(size_t, msl::Table const&, msl::Function const&, bool = false);

	void save(std::str const& file_name, std::str const& file_name_2 = std::str{}, bool exit = true);

	void transport_save(std::str const& file_name, bool exit = true);

	size_t get_index(msl::RefConst ref) { return &ref - &(*this)[0]; }

	//===============//
	// Sum functions //
	//===============//

	// population.sum([](msl::RefConst ref) { return ref.x + ref.y; });
	double sum(msl::Lambda const&, char const* = "sum") const;

	// population.sum(msl::var::x);
	double sum(msl::Table const&, char const* = "sum") const;

	// population.sum_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                   [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double sum_if(msl::Lambda const&, msl::Lambda const&, char const* = "sum_if") const;

	// population.sum_if([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	double sum_if(msl::Lambda const&, msl::Table const&, char const* = "sum_if") const;

	// population.sum_if(msl::var::x, msl::var::female);
	double sum_if(msl::Table const&, msl::Table const&, char const* = "sum_if") const;

	// population.sum_if(msl::var::x, [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double sum_if(msl::Table const&, msl::Lambda const&, char const* = "sum_if") const;

	// population.weighted_sum([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::w);
	double weighted_sum(msl::Lambda const&, msl::Table const&, char const* = "weighted_sum") const;

	// population.weighted_sum(msl::var::x);
	double weighted_sum(msl::Table const&, msl::Table const&, char const* = "weighted_sum") const;

	// population.weighted_sum_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_sum_if(msl::Lambda const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_sum_if") const;

	// population.weighted_sum_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            msl::var::female,
	//                            msl::var::w);
	double weighted_sum_if(msl::Lambda const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_sum_if") const;

	// population.weighted_sum_if(msl::var::x,
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_sum_if(msl::Table const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_sum_if") const;

	// population.weighted_sum_if(msl::var::x, msl::var::female, msl::var::w);
	double weighted_sum_if(msl::Table const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_sum_if") const;

	//===============//
	// Avg functions //
	//===============//

	// population.avg([](msl::RefConst ref) { return ref.x + ref.y; });
	double avg(msl::Lambda const&, char const* = "avg") const;

	// population.avg(msl::var::x);
	double avg(msl::Table const&, char const* = "avg") const;

	// population.avg_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                   [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double avg_if(msl::Lambda const&, msl::Lambda const&, char const* = "avg_if") const;

	// population.avg_if([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	double avg_if(msl::Lambda const&, msl::Table const&, char const* = "avg_if") const;

	// population.avg_if(msl::var::x, msl::var::female);
	double avg_if(msl::Table const&, msl::Table const&, char const* = "avg_if") const;

	// population.avg_if(msl::var::x, [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double avg_if(msl::Table const&, msl::Lambda const&, char const* = "avg_if") const;

	// population.weighted_avg([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::w);
	double weighted_avg(msl::Lambda const&, msl::Table const&, char const* = "weighted_avg") const;

	// population.weighted_avg(msl::var::x);
	double weighted_avg(msl::Table const&, msl::Table const&, char const* = "weighted_avg") const;

	// population.weighted_avg_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_avg_if(msl::Lambda const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_avg_if") const;

	// population.weighted_avg_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            msl::var::female,
	//                            msl::var::w);
	double weighted_avg_if(msl::Lambda const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_avg_if") const;

	// population.weighted_avg_if(msl::var::x,
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_avg_if(msl::Table const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_avg_if") const;

	// population.weighted_avg_if(msl::var::x, msl::var::female, msl::var::w);
	double weighted_avg_if(msl::Table const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_avg_if") const;

	//===============//
	// Std functions //
	//===============//

	// population.std([](msl::RefConst ref) { return ref.x + ref.y; });
	double std(msl::Lambda const&, char const* = "std") const;

	// population.std(msl::var::x);
	double std(msl::Table const&, char const* = "std") const;

	// population.std_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                   [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double std_if(msl::Lambda const&, msl::Lambda const&, char const* = "std_if") const;

	// population.std_if([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	double std_if(msl::Lambda const&, msl::Table const&, char const* = "std_if") const;

	// population.std_if(msl::var::x, msl::var::female);
	double std_if(msl::Table const&, msl::Table const&, char const* = "std_if") const;

	// population.std_if(msl::var::x, [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double std_if(msl::Table const&, msl::Lambda const&, char const* = "std_if") const;

	// population.weighted_std([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::w);
	double weighted_std(msl::Lambda const&, msl::Table const&, char const* = "weighted_std") const;

	// population.weighted_std(msl::var::x);
	double weighted_std(msl::Table const&, msl::Table const&, char const* = "weighted_std") const;

	// population.weighted_std_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_std_if(msl::Lambda const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_std_if") const;

	// population.weighted_std_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                            msl::var::female,
	//                            msl::var::w);
	double weighted_std_if(msl::Lambda const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_std_if") const;

	// population.weighted_std_if(msl::var::x,
	//                            [](msl::RefConst ref) { return ref.female() && (ref.x > 0); },
	//                            msl::var::w);
	double weighted_std_if(msl::Table const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "weighted_std_if") const;

	// population.weighted_std_if(msl::var::x, msl::var::female, msl::var::w);
	double weighted_std_if(msl::Table const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "weighted_std_if") const;

	//===============//
	// Min functions //
	//===============//

	// population.min([](msl::RefConst ref) { return ref.x + ref.y; });
	double min(msl::Lambda const&, char const* = "min") const;
	msl::RefConst ref_min(msl::Lambda const&, char const* = "min") const;
	msl::Ref ref_min(msl::Lambda const&, char const* = "min");

	// population.min(msl::var::x);
	double min(msl::Table const&, char const* = "min") const;
	msl::RefConst ref_min(msl::Table const&, char const* = "min") const;
	msl::Ref ref_min(msl::Table const&, char const* = "min");

	// population.min_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                   [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double min_if(msl::Lambda const&, msl::Lambda const&, char const* = "min_if") const;
	msl::RefConst ref_min_if(msl::Lambda const&, msl::Lambda const&, char const* = "min_if") const;
	msl::Ref ref_min_if(msl::Lambda const&, msl::Lambda const&, char const* = "min_if");

	// population.min_if([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	double min_if(msl::Lambda const&, msl::Table const&, char const* = "min_if") const;
	msl::RefConst ref_min_if(msl::Lambda const&, msl::Table const&, char const* = "min_if") const;
	msl::Ref ref_min_if(msl::Lambda const&, msl::Table const&, char const* = "min_if");

	// population.min_if(msl::var::x, msl::var::female);
	double min_if(msl::Table const&, msl::Table const&, char const* = "min_if") const;
	msl::RefConst ref_min_if(msl::Table const&, msl::Table const&, char const* = "min_if") const;
	msl::Ref ref_min_if(msl::Table const&, msl::Table const&, char const* = "min_if");

	// population.min_if(msl::var::x, [](msl::RefConst ref) { return ref.female() && (ref.y > 0); });
	double min_if(msl::Table const&, msl::Lambda const&, char const* = "min_if") const;
	msl::RefConst ref_min_if(msl::Table const&, msl::Lambda const&, char const* = "min_if") const;
	msl::Ref ref_min_if(msl::Table const&, msl::Lambda const&, char const* = "min_if");

	//===============//
	// Max functions //
	//===============//

	// population.max([](msl::RefConst ref) { return ref.x + ref.y; });
	double max(msl::Lambda const&, char const* = "max") const;
	msl::RefConst ref_max(msl::Lambda const&, char const* = "max") const;
	msl::Ref ref_max(msl::Lambda const&, char const* = "max");

	// population.max(msl::var::x);
	double max(msl::Table const&, char const* = "max") const;
	msl::RefConst ref_max(msl::Table const&, char const* = "max") const;
	msl::Ref ref_max(msl::Table const&, char const* = "max");

	// population.max_if([](msl::RefConst ref) { return ref.x + ref.y; },
	//                   [](msl::RefConst ref) { return ref.female() && (ref.x > 0); });
	double max_if(msl::Lambda const&, msl::Lambda const&, char const* = "max_if") const;
	msl::RefConst ref_max_if(msl::Lambda const&, msl::Lambda const&, char const* = "max_if") const;
	msl::Ref ref_max_if(msl::Lambda const&, msl::Lambda const&, char const* = "max_if");

	// population.max_if([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	double max_if(msl::Lambda const&, msl::Table const&, char const* = "max_if") const;
	msl::RefConst ref_max_if(msl::Lambda const&, msl::Table const&, char const* = "max_if") const;
	msl::Ref ref_max_if(msl::Lambda const&, msl::Table const&, char const* = "max_if");

	// population.max_if(msl::var::x, msl::var::female);
	double max_if(msl::Table const&, msl::Table const&, char const* = "max_if") const;
	msl::RefConst ref_max_if(msl::Table const&, msl::Table const&, char const* = "max_if") const;
	msl::Ref ref_max_if(msl::Table const&, msl::Table const&, char const* = "max_if");

	// population.max_if(msl::var::x, [](msl::RefConst ref) { return ref.female() && (ref.y > 0); });
	double max_if(msl::Table const&, msl::Lambda const&, char const* = "max_if") const;
	msl::RefConst ref_max_if(msl::Table const&, msl::Lambda const&, char const* = "max_if") const;
	msl::Ref ref_max_if(msl::Table const&, msl::Lambda const&, char const* = "max_if");

	//=======================//
	// Percentiles functions //
	//=======================//

	// We do not specialised these function members for ''Table'' variables as the main cost is on the
	// sort computation. We assume the overhead of using lambda...

	// population.pctl([](msl::RefConst ref) { return ref.x + ref.y; });
	std::array<double, 101> pctl(msl::Lambda const&, char const* = "pctl") const;

	// population.pctl(msl::var:x);
	std::array<double, 101> pctl(msl::Table const& get, char const* f_name = "pctl") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		return pctl(get.flt_, f_name);
	}

	// population.pctl([](msl::RefConst ref) { return ref.x + ref.y; }, [](mls::RefConst ref) { return
	// ref.female();});
	std::array<double, 101> pctl_if(msl::Lambda const&,
	                                msl::Lambda const&,
	                                char const* = "pctl_if") const;

	// population.pctl([](msl::RefConst ref) { return ref.x + ref.y; }, msl::var::female);
	std::array<double, 101> pctl_if(msl::Lambda const& get,
	                                msl::Table const& filter,
	                                char const* f_name = "pctl_if") const
	{
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		return pctl_if(get, filter.flt_, f_name);
	}

	// population.pctl(msl::var::x, [](mls::RefConst ref) { return ref.female();});
	std::array<double, 101> pctl_if(msl::Table const& get,
	                                msl::Lambda const& filter,
	                                char const* f_name = "pctl_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		return pctl_if(get.flt_, filter, f_name);
	}

	// population.pctl(msl::var::x, msl::var::female);
	std::array<double, 101> pctl_if(msl::Table const& get,
	                                msl::Table const& filter,
	                                char const* f_name = "pctl_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		return pctl_if(get.flt_, filter.flt_, f_name);
	}

	std::array<double, 101> weighted_pctl(msl::Lambda const&,
	                                      msl::Table const&,
	                                      char const* = "weighted_pctl") const;

	std::array<double, 101> weighted_pctl(msl::Table const& get,
	                                      msl::Table const& weight,
	                                      char const* f_name = "weighted_pctl") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_pctl(get.flt_, weight, f_name);
	}

	std::array<double, 101> weighted_pctl_if(msl::Lambda const&,
	                                         msl::Lambda const&,
	                                         msl::Table const& weight,
	                                         char const* = "weighted_pctl_if") const;

	std::array<double, 101> weighted_pctl_if(msl::Lambda const& get,
	                                         msl::Table const& filter,
	                                         msl::Table const& weight,
	                                         char const* f_name = "weighted_pctl_if") const
	{
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_pctl_if(get, filter.flt_, weight, f_name);
	}
	std::array<double, 101> weighted_pctl_if(msl::Table const& get,
	                                         msl::Lambda const& filter,
	                                         msl::Table const& weight,
	                                         char const* f_name = "weighted_pctl_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_pctl_if(get.flt_, filter, weight, f_name);
	}
	std::array<double, 101> weighted_pctl_if(msl::Table const& get,
	                                         msl::Table const& filter,
	                                         msl::Table const& weight,
	                                         char const* f_name = "weighted_pctl_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_pctl_if(get.flt_, filter.flt_, weight, f_name);
	}

	double gini(msl::Lambda const&, char const* = "gini") const;

	double gini(msl::Table const& get, char const* f_name = "gini") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		return gini(get.flt_, f_name);
	}

	double gini_if(msl::Lambda const&, msl::Lambda const&, char const* = "gini_if") const;

	double gini_if(msl::Lambda const& get,
	               msl::Table const& filter,
	               char const* f_name = "gini_if") const
	{
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		return gini_if(get, filter.flt_, f_name);
	}

	double gini_if(msl::Table const& get,
	               msl::Lambda const& filter,
	               char const* f_name = "gini_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		return gini_if(get.flt_, filter, f_name);
	}

	double gini_if(msl::Table const& get,
	               msl::Table const& filter,
	               char const* f_name = "gini_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		return gini_if(get.flt_, filter.flt_, f_name);
	}

	double weighted_gini(msl::Lambda const&, msl::Table const&, char const* = "weighted_gini") const;

	double weighted_gini(msl::Table const& get,
	                     msl::Table const& weight,
	                     char const* f_name = "weighted_gini") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_gini(get.flt_, weight, f_name);
	}

	double weighted_gini_if(msl::Lambda const&,
	                        msl::Lambda const&,
	                        msl::Table const&,
	                        char const* = "weighted_gini_if") const;

	double weighted_gini_if(msl::Lambda const& get,
	                        msl::Table const& filter,
	                        msl::Table const& weight,
	                        char const* f_name = "weighted_gini_if") const
	{
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_gini_if(get, filter.flt_, weight, f_name);
	}
	double weighted_gini_if(msl::Table const& get,
	                        msl::Lambda const& filter,
	                        msl::Table const& weight,
	                        char const* f_name = "weighted_gini_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_gini_if(get.flt_, filter, weight, f_name);
	}
	double weighted_gini_if(msl::Table const& get,
	                        msl::Table const& filter,
	                        msl::Table const& weight,
	                        char const* f_name = "weighted_gini_if") const
	{
		if (get.flt_ == nullptr)
			throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
		if (filter.flt_ == nullptr)
			throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
		if (weight.flt_ == nullptr)
			throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };
		return weighted_gini_if(get.flt_, filter.flt_, weight, f_name);
	}

	// population.sum_by([](msl::RefConst ref){ return ref.x + ref.y ; }, msl::var::std_living_10) ;
	msl::Tabular sum_by(msl::Lambda const&, msl::Table const&, char const* = "sum_by") const;

	// population.sum_by(msl::var::x, msl::var::std_living_10) ;
	msl::Tabular sum_by(msl::Table const&, msl::Table const&, char const* = "sum_by") const;

	// population.sum_by([](msl::RefConst ref){ return ref.x + ref.y ; }, msl::var::std_living_10,
	// msl::var::weight) ;
	msl::Tabular weighted_sum_by(msl::Lambda const&,
	                             msl::Table const&,
	                             msl::Table const&,
	                             char const* = "weighted_sum_by") const;

	// population.sum_by(msl::var::x, msl::var::std_living_10, msl::var::weight) ;
	msl::Tabular weighted_sum_by(msl::Table const&,
	                             msl::Table const&,
	                             msl::Table const&,
	                             char const* = "weighted_sum_by") const;

	// population.sum_if_by([](msl::RefConst ref){ return ref.x + ref.y; },
	//                      [](msl::RefConst ref){ return ref.female() && (ref.z > 0); },
	//                      msl::var::std_living_10);
	msl::Tabular sum_if_by(msl::Lambda const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "sum_if_by") const;
	msl::Tabular sum_if_by(msl::Lambda const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "sum_if_by") const;
	msl::Tabular sum_if_by(msl::Table const&,
	                       msl::Lambda const&,
	                       msl::Table const&,
	                       char const* = "sum_if_by") const;
	msl::Tabular sum_if_by(msl::Table const&,
	                       msl::Table const&,
	                       msl::Table const&,
	                       char const* = "sum_if_by") const;

	// Weighted and filtered versions.
	msl::Tabular weighted_sum_if_by(msl::Lambda const&,
	                                msl::Lambda const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                char const* = "weighted_sum_if_by") const;
	msl::Tabular weighted_sum_if_by(msl::Lambda const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                char const* = "weighted_sum_if_by") const;
	msl::Tabular weighted_sum_if_by(msl::Table const&,
	                                msl::Lambda const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                char const* = "weighted_sum_if_by") const;
	msl::Tabular weighted_sum_if_by(msl::Table const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                msl::Table const&,
	                                char const* = "weighted_sum_if_by") const;

	std::vector<size_t> get_indexes(char const* = "get_indexes") const;
	std::vector<double> get_variable(msl::Lambda const&, char const* = "get_variable") const;
	std::vector<double> get_variable(msl::Table const&, char const* = "get_variable") const;
	std::vector<size_t> get_indexes_if(msl::Lambda const&, char const* = "get_indexes_if") const;
	std::vector<size_t> get_indexes_if(msl::Table const&, char const* = "get_indexes_if") const;
	std::vector<double> get_variable_if(msl::Lambda const&,
	                                    msl::Lambda const&,
	                                    char const* = "get_variable_if") const;
	std::vector<double> get_variable_if(msl::Lambda const&,
	                                    msl::Table const&,
	                                    char const* = "get_variable_if") const;
	std::vector<double> get_variable_if(msl::Table const&,
	                                    msl::Lambda const&,
	                                    char const* = "get_variable_if") const;
	std::vector<double> get_variable_if(msl::Table const&,
	                                    msl::Table const&,
	                                    char const* = "get_variable_if") const;
	void set_variable(msl::Table const&,
	                  std::vector<size_t> const&,
	                  std::vector<double> const&,
	                  char const* = "set_variable");

// Declare the ''for_each_*” member function, the ''weighted_sum_*_head” and the
// ''weighted_sum_*_head_by”.
#define LNK(prm)                                                                                   \
                                                                                                   \
	/* Like auto population.for_each_family(...) */                                                  \
	void for_each_##prm(msl::Function function)                                                      \
	{                                                                                                \
		return this->for_each_if(function, msl::var::prm##_head);                                      \
	}                                                                                                \
                                                                                                   \
	/* Like double population.sum_family(...) */                                                     \
	double sum_##prm(msl::Lambda const& get)                                                         \
	{                                                                                                \
		return this->sum_if(get, msl::var::prm##_head);                                                \
	}                                                                                                \
                                                                                                   \
	/* Like double population.sum_family(...) */                                                     \
	double sum_##prm(msl::Table const& get)                                                          \
	{                                                                                                \
		return this->sum_if(get, msl::var::prm##_head);                                                \
	}                                                                                                \
                                                                                                   \
	/* Like double population.weighted_sum_family(...) */                                            \
	double weighted_sum_##prm(msl::Lambda const& get)                                                \
	{                                                                                                \
		return this->weighted_sum_if(get, msl::var::prm##_head, msl::var::prm##_weight);               \
	}                                                                                                \
                                                                                                   \
	/* Like double population.weighted_sum_family(...) */                                            \
	double weighted_sum_##prm(msl::Table const& get)                                                 \
	{                                                                                                \
		return this->weighted_sum_if(get, msl::var::prm##_head, msl::var::prm##_weight);               \
	}                                                                                                \
                                                                                                   \
	/* Like msl::Tabular population.weighted_sum_family_by(...) */                                   \
	msl::Tabular weighted_sum_##prm##_by(msl::Lambda const& get, msl::Table const& by)               \
	{                                                                                                \
		return this->weighted_sum_if_by(get, msl::var::prm##_head, by, msl::var::prm##_weight);        \
	}                                                                                                \
                                                                                                   \
	/* Like msl::Tabular population.weighted_sum_family_by(...) */                                   \
	msl::Tabular weighted_sum_##prm##_by(msl::Table const& get, msl::Table const& by)                \
	{                                                                                                \
		return this->weighted_sum_if_by(get, msl::var::prm##_head, by, msl::var::prm##_weight);        \
	}

#define FLT(prm)         /* nothing */
#define FLG(prm)         /* nothing */
#define CTG(prm, format) /* nothing */
#define PTR(prm)         /* nothing */
#include "msl-variables.cpp"
#undef PTR
#undef CTG
#undef FLG
#undef FLT
#undef LNK
}; // struct Population
