// Using a lambda: population.max([](msl::RefConst ref) { return ref.x + ref.y; })
namespace internals {
struct CmpGreatLambda
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref2);
		}
		if (!std::isnan(x1)) {
			if (!std::isnan(x2))
				// x1 < x2
				return x1 < x2;
			else
				// x1 < not alive2 -> false
				return false;
		} else
			// not alive1 < x2 -> true
			// not alive1 < not alive2 -> true
			return true;
	}
	msl::Lambda get_;
	char const* f_name_;
};
// Using a variable: population.max(msl::var::x)
struct CmpGreatRawPtr
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref2);
		}
		if (!std::isnan(x1)) {
			if (!std::isnan(x2))
				// x1 < x2
				return x1 < x2;
			else
				// x1 < not alive2 -> false
				return false;
		} else
			// not alive1 < x2 -> true
			// not alive1 < not alive2 -> true
			return true;
	}
	msl::Flt get_;
	char const* f_name_;
	char const* v_name_;
};

struct CmpGreatIfLambdaLambda
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		bool selected1, selected2;
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref1);
			selected1 = filter_(ref1);
			if (std::isnan(selected1))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref2);
			selected2 = filter_(ref2);
			if (std::isnan(selected2))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref2);
		}
		if (!std::isnan(x1)) {
			if (selected1) {
				if (!std::isnan(x2)) {
					if (selected2)
						// x1 < x2
						return x1 < x2;
					else
						// x1 < not selected2 -> false
						return false;
				} else { // alive2
					// x1 < not selected -> false
					return false;
				}
			} else { // selected1
				if (!std::isnan(x2))
					// not selected1 < x2 -> true
					// not selected1 < not selected2 -> true
					return true;
			}
		} else { // alive1
			// not alive1 < alive2 -> true
			// not alive1 < not alive2 -> true
			return true;
		}
		assert("Not reached");
		return false;
	}
	msl::Lambda get_;
	msl::Lambda filter_;
	char const* f_name_;
};

struct CmpGreatIfLambdaRawPtr
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		bool selected1, selected2;
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref1);
			selected1 = filter_(ref1);
			if (std::isnan(selected1))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref2);
			selected2 = filter_(ref2);
			if (std::isnan(selected2))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref2);
		}
		if (!std::isnan(x1)) {
			if (selected1) {
				if (!std::isnan(x2)) {
					if (selected2)
						// x1 < x2
						return x1 < x2;
					else
						// x1 < not selected2 -> false
						return false;
				} else { // alive2
					// x1 < not selected -> false
					return false;
				}
			} else { // selected1
				if (!std::isnan(x2))
					// not selected1 < x2 -> true
					// not selected1 < not selected2 -> true
					return true;
			}
		} else { // alive1
			// not alive1 < alive2 -> true
			// not alive1 < not alive2 -> true
			return true;
		}
		assert("Not reached");
		return false;
	}
	msl::Lambda get_;
	msl::Flt filter_;
	char const* f_name_;
	char const* if_name_;
};

struct CmpGreatIfRawPtrLambda
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		bool selected1, selected2;
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref1);
			selected1 = filter_(ref1);
			if (std::isnan(selected1))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref2);
			selected2 = filter_(ref2);
			if (std::isnan(selected2))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref2);
		}
		if (!std::isnan(x1)) {
			if (selected1) {
				if (!std::isnan(x2)) {
					if (selected2)
						// x1 < x2
						return x1 < x2;
					else
						// x1 < not selected2 -> false
						return false;
				} else { // alive2
					// x1 < not selected -> false
					return false;
				}
			} else { // selected1
				if (!std::isnan(x2))
					// not selected1 < x2 -> true
					// not selected1 < not selected2 -> true
					return true;
			}
		} else { // alive1
			// not alive1 < alive2 -> true
			// not alive1 < not alive2 -> true
			return true;
		}
		assert("Not reached");
		return false;
	}
	msl::Flt get_;
	msl::Lambda filter_;
	char const* f_name_;
	char const* v_name_;
};

struct CmpGreatIfRawPtrRawPtr
{
	bool operator()(msl::RefConst ref1, msl::RefConst ref2)
	{
		double x1{ std::numeric_limits<double>::quiet_NaN() };
		double x2{ std::numeric_limits<double>::quiet_NaN() };
		bool selected1, selected2;
		if (ref1.alive()) {
			x1 = get_(ref1);
			if (std::isnan(x1))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref1);
			selected1 = filter_(ref1);
			if (std::isnan(selected1))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref1);
		}
		if (ref2.alive()) {
			x2 = get_(ref2);
			if (std::isnan(x2))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref2);
			selected2 = filter_(ref2);
			if (std::isnan(selected2))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref2);
		}
		if (!std::isnan(x1)) {
			if (selected1) {
				if (!std::isnan(x2)) {
					if (selected2)
						// x1 < x2
						return x1 < x2;
					else
						// x1 < not selected2 -> false
						return false;
				} else { // alive2
					// x1 < not selected -> false
					return false;
				}
			} else { // selected1
				if (!std::isnan(x2))
					// not selected1 < x2 -> true
					// not selected1 < not selected2 -> true
					return true;
			}
		} else { // alive1
			// not alive1 < alive2 -> true
			// not alive1 < not alive2 -> true
			return true;
		}
		assert("Not reached");
		return false;
	}
	msl::Flt get_;
	msl::Flt filter_;
	char const* f_name_;
	char const* v_name_;
	char const* if_name_;
};
} // namespace internals

//===============//
// Min functions //
//===============//

double
msl::Population::max(msl::Lambda const& get, char const* f_name) const
{
	return get(*std::max_element(std::execution::par,
	                             this->cbegin(),
	                             this->cend(),
	                             msl::internals::CmpGreatLambda{ get, f_name }));
}
msl::RefConst
msl::Population::ref_max(msl::Lambda const& get, char const* f_name) const
{
	return *std::max_element(std::execution::par,
	                         this->cbegin(),
	                         this->cend(),
	                         msl::internals::CmpGreatLambda{ get, f_name });
}
msl::Ref
msl::Population::ref_max(msl::Lambda const& get, char const* f_name)
{
	return const_cast<msl::Ref>(const_cast<msl::Population const*>(this)->ref_max(get, f_name));
}
double
msl::Population::max(msl::Table const& get, char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("max", get.name_()) };
	return get.flt_(
	  *std::max_element(std::execution::par,
	                    this->cbegin(),
	                    this->cend(),
	                    msl::internals::CmpGreatRawPtr{ get.flt_, f_name, get.name_() }));
}
msl::RefConst
msl::Population::ref_max(msl::Table const& get, char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("ref_max", get.name_()) };
	return *std::max_element(std::execution::par,
	                         this->cbegin(),
	                         this->cend(),
	                         msl::internals::CmpGreatRawPtr{ get.flt_, f_name, get.name_() });
}
msl::Ref
msl::Population::ref_max(msl::Table const& get, char const* f_name)
{
	return const_cast<msl::Ref>(const_cast<msl::Population const*>(this)->ref_max(get, f_name));
}
double
msl::Population::max_if(msl::Lambda const& get, msl::Lambda const& filter, char const* f_name) const
{
	return get(*std::max_element(std::execution::par,
	                             this->cbegin(),
	                             this->cend(),
	                             msl::internals::CmpGreatIfLambdaLambda{ get, filter, f_name }));
}
msl::RefConst
msl::Population::ref_max_if(msl::Lambda const& get,
                            msl::Lambda const& filter,
                            char const* f_name) const
{
	return *std::max_element(std::execution::par,
	                         this->cbegin(),
	                         this->cend(),
	                         msl::internals::CmpGreatIfLambdaLambda{ get, filter, f_name });
}
msl::Ref
msl::Population::ref_max_if(msl::Lambda const& get, msl::Lambda const& filter, char const* f_name)
{
	return const_cast<msl::Ref>(
	  const_cast<msl::Population const*>(this)->ref_max_if(get, filter, f_name));
}
double
msl::Population::max_if(msl::Lambda const& get, msl::Table const& filter, char const* f_name) const
{
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("max_if", filter.name_()) };
	return get(*std::max_element(
	  std::execution::par,
	  this->cbegin(),
	  this->cend(),
	  msl::internals::CmpGreatIfLambdaRawPtr{ get, filter.flt_, f_name, filter.name_() }));
}
msl::RefConst
msl::Population::ref_max_if(msl::Lambda const& get,
                            msl::Table const& filter,
                            char const* f_name) const
{
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("ref_max_if", filter.name_()) };
	return *std::max_element(
	  std::execution::par,
	  this->cbegin(),
	  this->cend(),
	  msl::internals::CmpGreatIfLambdaRawPtr{ get, filter.flt_, f_name, filter.name_() });
}
msl::Ref
msl::Population::ref_max_if(msl::Lambda const& get, msl::Table const& filter, char const* f_name)
{
	return const_cast<msl::Ref>(
	  const_cast<msl::Population const*>(this)->ref_max_if(get, filter, f_name));
}
double
msl::Population::max_if(msl::Table const& get, msl::Lambda const& filter, char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("max_if", get.name_()) };
	return get.flt_(*std::max_element(
	  std::execution::par,
	  this->cbegin(),
	  this->cend(),
	  msl::internals::CmpGreatIfRawPtrLambda{ get.flt_, filter, f_name, get.name_() }));
}
msl::RefConst
msl::Population::ref_max_if(msl::Table const& get,
                            msl::Lambda const& filter,
                            char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("ref_max_if", get.name_()) };
	return *std::max_element(
	  std::execution::par,
	  this->cbegin(),
	  this->cend(),
	  msl::internals::CmpGreatIfRawPtrLambda{ get.flt_, filter, f_name, get.name_() });
}
msl::Ref
msl::Population::ref_max_if(msl::Table const& get, msl::Lambda const& filter, char const* f_name)
{
	return const_cast<msl::Ref>(
	  const_cast<msl::Population const*>(this)->ref_max_if(get, filter, f_name));
}
double
msl::Population::max_if(msl::Table const& get, msl::Table const& filter, char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("max_if", get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("max_if", filter.name_()) };
	return get.flt_(
	  *std::max_element(std::execution::par,
	                    this->cbegin(),
	                    this->cend(),
	                    msl::internals::CmpGreatIfRawPtrRawPtr{
	                      get.flt_, filter.flt_, f_name, get.name_(), filter.name_() }));
}
msl::RefConst
msl::Population::ref_max_if(msl::Table const& get,
                            msl::Table const& filter,
                            char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("ref_max_if", get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("ref_max_if", filter.name_()) };
	return *std::max_element(std::execution::par,
	                         this->cbegin(),
	                         this->cend(),
	                         msl::internals::CmpGreatIfRawPtrRawPtr{
	                           get.flt_, filter.flt_, f_name, get.name_(), filter.name_() });
}
msl::Ref
msl::Population::ref_max_if(msl::Table const& get, msl::Table const& filter, char const* f_name)
{
	return const_cast<msl::Ref>(
	  const_cast<msl::Population const*>(this)->ref_max_if(get, filter, f_name));
}
