// The “std::array<double, 4>” objet is returned by dereferencing some iterator. If the dd.first
// value is a missing one, the individual is a no alive one. The dd.first is the value for the
// variable of interest. It is not the same as AvgReduceIterator.
namespace internals {
struct StdReduceOperator
{
	std::array<double, 4> operator()(msl::DoubleDouble dd, std::array<double, 4> array)
	{
		auto const x{ dd.first };
		if (!std::isnan(x)) {
			auto const w{ dd.second };
			array[0] += w * x * x;
			array[1] += w * x;
			array[2] += w;
			++array[3];
		}
		return array;
	}
	std::array<double, 4> operator()(std::array<double, 4> array, msl::DoubleDouble dd)
	{
		auto const x{ dd.first };
		if (!std::isnan(x)) {
			auto const w{ dd.second };
			array[0] += w * x * x;
			array[1] += w * x;
			array[2] += w;
			++array[3];
		}
		return array;
	}
	// The two arguments must be merged.
	std::array<double, 4> operator()(std::array<double, 4> array1, std::array<double, 4> array2)
	{
		array1[0] += array2[0];
		array1[1] += array2[1];
		array1[2] += array2[2];
		array1[3] += array2[3];
		return array1;
	}
	// Start a task.
	std::array<double, 4> operator()(msl::DoubleDouble dd1, msl::DoubleDouble dd2)
	{
		auto const x1{ dd1.first };
		auto const x2{ dd2.first };

		if (!std::isnan(x1)) {
			if (!std::isnan(x2)) {
				auto const w1{ dd1.second };
				auto const w2{ dd2.second };
				return { w1 * x1 * x1 + w2 * x2 * x2, w1 * x1 + w2 * x2, w1 + w2, 2 };
			} else {
				auto const w1{ dd1.second };
				return { w1 * x1 * x1, w1 * x1, w1, 1 };
			}
		} else {
			if (!std::isnan(x2)) {
				auto const w2{ dd2.second };
				return { w2 * x2 * x2, w2 * x2, w2, 1 };
			} else
				return { double{}, double{}, double{}, double{} };
		}
	}
};
} // namespace internals
double
msl::Population::std_if(msl::Lambda const& get, msl::Lambda const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to std.
		msl::Lambda get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
	}; // struct MyIterator

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(), get, filter, f_name },
		MyIterator{ static_cast<msl::PopBase const&>(*this).cend(), get, filter, f_name },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::std_if(msl::Lambda const& get, msl::Table const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_, if_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to std.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
	}; // struct MyIterator

	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("std_if", filter.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cbegin(), get, filter.flt_, f_name, filter.name_() },
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cend(), get, filter.flt_, f_name, filter.name_() },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::std_if(msl::Table const& get, msl::Lambda const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_, v_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cbegin(), get.flt_, filter, f_name, get.name_() },
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cend(), get.flt_, filter, f_name, get.name_() },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::std_if(msl::Table const& get, msl::Table const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_, v_name_, if_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("std_if", get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("std_if", filter.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get.flt_,
		                                        filter.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        filter.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get.flt_,
		                                        filter.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        filter.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::std(msl::Lambda const& get, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, f_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to std.
		msl::Lambda get_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
	}; // struct MyIterator

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(), get, f_name },
		MyIterator{ static_cast<msl::PopBase const&>(*this).cend(), get, f_name },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::std(msl::Table const& get, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, double{ 1 } };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, f_name_, v_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(), get.flt_, f_name, get.name_() },
		MyIterator{ static_cast<msl::PopBase const&>(*this).cend(), get.flt_, f_name, get.name_() },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	return std::sqrt(s2 / (n - 1) - (s / n) * (s / (n - 1)));
}
double
msl::Population::weighted_std_if(msl::Lambda const& get,
                                 msl::Lambda const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to std.
		msl::Lambda get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get,
		                                        filter,
		                                        weight.flt_,
		                                        f_name,
		                                        weight.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get,
		                                        filter,
		                                        weight.flt_,
		                                        f_name,
		                                        weight.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
double
msl::Population::weighted_std_if(msl::Lambda const& get,
                                 msl::Table const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, if_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to std.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get,
		                                        filter.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        filter.name_(),
		                                        weight.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get,
		                                        filter.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        filter.name_(),
		                                        weight.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
double
msl::Population::weighted_std_if(msl::Table const& get,
                                 msl::Lambda const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, v_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get.flt_,
		                                        filter,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        weight.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get.flt_,
		                                        filter,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        weight.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
double
msl::Population::weighted_std_if(msl::Table const& get,
                                 msl::Table const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			auto const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns NaN.
			if (!crit)
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			auto const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return {
				BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, v_name_, if_name_, w_name_
			};
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get.flt_,
		                                        filter.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        filter.name_(),
		                                        weight.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get.flt_,
		                                        filter.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        filter.name_(),
		                                        weight.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
double
msl::Population::weighted_std(msl::Lambda const& get,
                              msl::Table const& weight,
                              char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, weight_, f_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(
		std::execution::par_unseq,
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cbegin(), get, weight.flt_, f_name, weight.name_() },
		MyIterator{
		  static_cast<msl::PopBase const&>(*this).cend(), get, weight.flt_, f_name, weight.name_() },
		std::array<double, 4>{},
		msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
double
msl::Population::weighted_std(msl::Table const& get,
                              msl::Table const& weight,
                              char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		msl::DoubleDouble operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns NaN, then a no-op for the
			// msl::internals::StdReduceOperator.
			if (!ref.alive())
				return { std::numeric_limits<double>::quiet_NaN(), 0 };
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			auto const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the input of the msl::internals::StdReduceOperator.
			return { x, w };
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, weight_, f_name_, v_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		msl::DoubleDouble operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to std.
		msl::Flt get_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, get.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	auto const array{ std::reduce(std::execution::par_unseq,
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cbegin(),
		                                        get.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        weight.name_() },
		                            MyIterator{ static_cast<msl::PopBase const&>(*this).cend(),
		                                        get.flt_,
		                                        weight.flt_,
		                                        f_name,
		                                        get.name_(),
		                                        weight.name_() },
		                            std::array<double, 4>{},
		                            msl::internals::StdReduceOperator{}) };
	auto const n{ array[3] };
	if (n == 0)
		return std::numeric_limits<double>::quiet_NaN();
	if (n == 1)
		return double{};
	auto const s2{ array[0] };
	auto const s{ array[1] };
	auto const sw{ array[2] };
	return std::sqrt((s2 - s * (s / sw)) / (n - 1) * n / sw);
}
