namespace internals {
msl::Tabular
end_of_the_sum_by_job(std::vector<std::array<double, CTG_NR>> const& sums, msl::Ctg ctg)
{
	// The Tabular objet is a std::map for only the by values used.
	msl::Tabular rvo;
	// No format for the CTG member data...
	if (msl::global_formats.find(ctg) == msl::global_formats.end()) {
		// For each array in the vector.
		for (auto const [i, sum] : msl::enumerate(sums)) {
			// For each category.
			for (auto const [category, value] : msl::enumerate(sum)) {
				if (value != double{}) {
					auto const iterator{ rvo.find(category) };
					if (iterator == rvo.end())
						rvo[category] = { "", value };
					else
						iterator->second.second += value;
				}
			}
		}
		return rvo;
	}
	auto& [format_name, format_map]{ msl::global_formats[ctg] };
	for (auto const [i, sum] : msl::enumerate(sums)) {
		for (auto const [category, value] : msl::enumerate(sum)) {
			if (value != double{}) {
				auto const iterator{ rvo.find(category) };
				if (iterator == rvo.end())
					rvo[category] = { format_map[uint8_t(category)], value };
				else
					iterator->second.second += value;
			}
		}
	}
	return rvo;
}
} // namespace internals
msl::Tabular
msl::Population::sum_by(msl::Lambda const& get, msl::Table const& by, char const* f_name) const
{
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				double const x{ get(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(x))
					msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
				// Compute the effective sum.
				sum[by.ctg_(ref)] += x;
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::sum_by(msl::Table const& get, msl::Table const& by, char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const x{ get.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(x))
					msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
				// Compute the effective sum.
				sum[by.ctg_(ref)] += x;
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::sum_if_by(msl::Lambda const& get,
                           msl::Lambda const& filter,
                           msl::Table const& by,
                           char const* f_name) const
{
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				double const crit{ filter(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, "<lambda>", ref);
				// Only if the criterion is satisfied.
				if (crit) {
					double const x{ get(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::sum_if_by(msl::Table const& get,
                           msl::Lambda const& filter,
                           msl::Table const& by,
                           char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				double const crit{ filter(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, "<lambda>", ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const x{ get.flt_(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::sum_if_by(msl::Lambda const& get,
                           msl::Table const& filter,
                           msl::Table const& by,
                           char const* f_name) const
{
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const crit{ filter.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, filter.name_(), ref);
				// Only if the criterion is satisfied.
				if (crit) {
					double const x{ get(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::sum_if_by(msl::Table const& get,
                           msl::Table const& filter,
                           msl::Table const& by,
                           char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const crit{ filter.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, filter.name_(), ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const x{ get.flt_(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_by(msl::Lambda const& get,
                                 msl::Table const& by,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const w{ weight.flt_(ref) };
				if (std::isnan(w))
					msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
				if (w < 0)
					msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
				double const x{ get(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(x))
					msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
				// Compute the effective sum.
				sum[by.ctg_(ref)] += w * x;
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_by(msl::Table const& get,
                                 msl::Table const& by,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const w{ weight.flt_(ref) };
				if (std::isnan(w))
					msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
				if (w < 0)
					msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
				float const x{ get.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(x))
					msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
				// Compute the effective sum.
				sum[by.ctg_(ref)] += w * x;
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_if_by(msl::Lambda const& get,
                                    msl::Lambda const& filter,
                                    msl::Table const& by,
                                    msl::Table const& weight,
                                    char const* f_name) const
{
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				double const crit{ filter(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, "<lambda>", ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const w{ weight.flt_(ref) };
					if (std::isnan(w))
						msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
					if (w < 0)
						msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
					double const x{ get(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += w * x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_if_by(msl::Lambda const& get,
                                    msl::Table const& filter,
                                    msl::Table const& by,
                                    msl::Table const& weight,
                                    char const* f_name) const
{
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const crit{ filter.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, filter.name_(), ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const w{ weight.flt_(ref) };
					if (std::isnan(w))
						msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
					if (w < 0)
						msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
					double const x{ get(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, "<lambda>", ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += w * x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_if_by(msl::Table const& get,
                                    msl::Lambda const& filter,
                                    msl::Table const& by,
                                    msl::Table const& weight,
                                    char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				double const crit{ filter(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, "<lambda>", ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const w{ weight.flt_(ref) };
					if (std::isnan(w))
						msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
					if (w < 0)
						msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
					float const x{ get.flt_(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += w * x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
msl::Tabular
msl::Population::weighted_sum_if_by(msl::Table const& get,
                                    msl::Table const& filter,
                                    msl::Table const& by,
                                    msl::Table const& weight,
                                    char const* f_name) const
{
	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (by.ctg_ == nullptr)
		throw msl::Exception{ error_as_by_variable(f_name, by.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The result for each thread.
	std::vector<std::array<double, CTG_NR>> sums(msl::THREADS_NR);
	// The task to execute.
	auto const task{ [&](size_t i) {
		std::array<double, CTG_NR> sum{};
		// Each thread works with a msl::THREADS_NR shift.
		for (size_t j{ i }; j < this->size(); j += msl::THREADS_NR) {
			msl::RefConst ref{ (*this)[j] };
			// Only for alive individual.
			if (ref.alive()) {
				float const crit{ filter.flt_(ref) };
				// Check if the value is not a missing one. Display an error and exit.
				if (std::isnan(crit))
					msl::error_get_nan_if_variable(f_name, filter.name_(), ref);
				// Only if the criterion is satisfied.
				if (crit) {
					float const w{ weight.flt_(ref) };
					if (std::isnan(w))
						msl::error_get_nan_weight_variable(f_name, weight.name_(), ref);
					if (w < 0)
						msl::error_get_negative_weight_variable(f_name, weight.name_(), ref);
					float const x{ get.flt_(ref) };
					// Check if the value is not a missing one. Display an error and exit.
					if (std::isnan(x))
						msl::error_get_nan_interest_variable(f_name, get.name_(), ref);
					// Compute the effective sum.
					sum[by.ctg_(ref)] += w * x;
				}
			}
		}
		// Save the result at the right place.
		sums[i] = sum;
	} };
	// Start each thread.
	for (size_t i{}; i < THREADS_NR; ++i)
		threads.emplace_back(task, i);
	// Wait for the exit of each thread.
	msl::join(threads);

	return msl::internals::end_of_the_sum_by_job(sums, by.ctg_);
}
