double
msl::Population::sum_if(msl::Lambda const& get, msl::Lambda const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to sum.
		msl::Lambda get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
	}; // struct MyIterator

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get, filter, f_name },
	                   MyIterator{ this->cend(), get, filter, f_name },
	                   double{});
}
double
msl::Population::sum_if(msl::Lambda const& get, msl::Table const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_, if_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to sum.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
	}; // struct MyIterator

	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("sum_if", filter.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get, filter.flt_, f_name, filter.name_() },
	                   MyIterator{ this->cend(), get, filter.flt_, f_name, filter.name_() },
	                   double{});
}
double
msl::Population::sum_if(msl::Table const& get, msl::Lambda const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get.flt_, filter, f_name, get.name_() },
	                   MyIterator{ this->cend(), get.flt_, filter, f_name, get.name_() },
	                   double{});
}
double
msl::Population::sum_if(msl::Table const& get, msl::Table const& filter, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, f_name_, v_name_, if_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable("sum_if", get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable("sum_if", filter.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(
	  std::execution::par_unseq,
	  MyIterator{ this->cbegin(), get.flt_, filter.flt_, f_name, get.name_(), filter.name_() },
	  MyIterator{ this->cend(), get.flt_, filter.flt_, f_name, get.name_(), filter.name_() },
	  double{});
}
double
msl::Population::sum(msl::Lambda const& get, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, f_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to sum.
		msl::Lambda get_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
	}; // struct MyIterator

	// A thread for each core.
	auto threads{ msl::reserve<std::thread>(msl::THREADS_NR) };
	// The total range is splitted in chunks of step width.
	auto const step{ (this->size() - 1) / msl::THREADS_NR + 1 };
	// A vector of sums for each core.
	std::vector<double> sums(msl::THREADS_NR, 0);
	// The task to execute on each chunk.
	auto const task{ [&](size_t i, size_t begin, size_t end) {
		sums[i] = std::accumulate(MyIterator{ this->cbegin() + begin, get, f_name },
		                          MyIterator{ this->cbegin() + end, get, f_name },
		                          double{});
	} };
	// Parallelised execution of the task.
	size_t begin{};
	for (size_t i{}; i < msl::THREADS_NR; ++i) {
		auto const end{ std::min(begin + step, this->size()) };
		threads.emplace_back(task, i, begin, end);
		begin = end;
	}
	msl::join(threads);
	return std::accumulate(sums.cbegin(), sums.cend(), double{});
}
double
msl::Population::sum(msl::Table const& get, char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, f_name_, v_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };

	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get.flt_, f_name, get.name_() },
	                   MyIterator{ this->cend(), get.flt_, f_name, get.name_() },
	                   double{});
}
double
msl::Population::weighted_sum_if(msl::Lambda const& get,
                                 msl::Lambda const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to sum.
		msl::Lambda get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get, filter, weight.flt_, f_name, weight.name_() },
	                   MyIterator{ this->cend(), get, filter, weight.flt_, f_name, weight.name_() },
	                   double{});
}
double
msl::Population::weighted_sum_if(msl::Lambda const& get,
                                 msl::Table const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, if_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The lambda with returns the value to sum.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(
	  std::execution::par_unseq,
	  MyIterator{
	    this->cbegin(), get, filter.flt_, weight.flt_, f_name, filter.name_(), weight.name_() },
	  MyIterator{
	    this->cend(), get, filter.flt_, weight.flt_, f_name, filter.name_(), weight.name_() },
	  double{});
}
double
msl::Population::weighted_sum_if(msl::Table const& get,
                                 msl::Lambda const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		double operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			double const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, "<lambda>", ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, v_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The lambda with returns the value of the criterion.
		msl::Lambda filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(
	  std::execution::par_unseq,
	  MyIterator{
	    this->cbegin(), get.flt_, filter, weight.flt_, f_name, get.name_(), weight.name_() },
	  MyIterator{ this->cend(), get.flt_, filter, weight.flt_, f_name, get.name_(), weight.name_() },
	  double{});
}
double
msl::Population::weighted_sum_if(msl::Table const& get,
                                 msl::Table const& filter,
                                 msl::Table const& weight,
                                 char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		float operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			auto const crit{ filter_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(crit))
				msl::error_get_nan_if_variable(f_name_, if_name_, ref);
			// The criterion is not satisfied,  the iterator returns 0, then a no-op for the sum.
			if (!crit)
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			auto const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return {
				BaseIterator{ *this } + n, get_, filter_, weight_, f_name_, v_name_, if_name_, w_name_
			};
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The raw pointer on a free function with returns the value of the criterion.
		msl::Flt filter_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the if variable.
		char const* if_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (filter.flt_ == nullptr)
		throw msl::Exception{ error_as_if_variable(f_name, filter.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(),
	                               get.flt_,
	                               filter.flt_,
	                               weight.flt_,
	                               f_name,
	                               get.name_(),
	                               filter.name_(),
	                               weight.name_() },
	                   MyIterator{ this->cend(),
	                               get.flt_,
	                               filter.flt_,
	                               weight.flt_,
	                               f_name,
	                               get.name_(),
	                               filter.name_(),
	                               weight.name_() },
	                   double{});
}
double
msl::Population::weighted_sum(msl::Lambda const& get,
                              msl::Table const& weight,
                              char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		float operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			double const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, "<lambda>", ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, weight_, f_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Lambda get_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, weight.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(std::execution::par_unseq,
	                   MyIterator{ this->cbegin(), get, weight.flt_, f_name, weight.name_() },
	                   MyIterator{ this->cend(), get, weight.flt_, f_name, weight.name_() },
	                   double{});
}
double
msl::Population::weighted_sum(msl::Table const& get,
                              msl::Table const& weight,
                              char const* f_name) const
{
	using BaseIterator = msl::PopBase::const_iterator;
	struct MyIterator : public BaseIterator
	{
		// The dereferencing of the iterator do not return the objet as the usual iterator.
		float operator*() const
		{
			msl::RefConst ref{ (BaseIterator::operator*)() };
			// If the individual is not a alive one, the iterator returns 0, then a no-op for the sum.
			if (!ref.alive())
				return 0;
			float const w{ weight_(ref) };
			if (std::isnan(w))
				msl::error_get_nan_weight_variable(f_name_, w_name_, ref);
			if (w < 0)
				msl::error_get_negative_weight_variable(f_name_, w_name_, ref);
			auto const x{ get_(ref) };
			// Check if the value is not a missing one. Display an error and exit.
			if (std::isnan(x))
				msl::error_get_nan_interest_variable(f_name_, v_name_, ref);
			// Return the effective value.
			return w * x;
		}
		// The reduce algorithm migth want to split the range like that
		//      middle = begin + (end-begin)/2
		// The operator- is happy with the base class operator- but the operator+ must be
		// redefined.
		MyIterator operator+(BaseIterator::difference_type n) const
		{
			// Get the base class iterator.
			return { BaseIterator{ *this } + n, get_, weight_, f_name_, v_name_, w_name_ };
		}
		// The reduce algorithm migth want direct acces to element.
		double operator[](BaseIterator::difference_type n) const { return *(*this + n); }
		// The raw pointer on a free function with returns the value to sum.
		msl::Flt get_;
		// The raw pointer on a free function with returns the weight.
		msl::Flt weight_;
		// The raw pointer on a null terminated string witch gives the name of the function.
		char const* f_name_;
		// The raw pointer on a null terminated string witch gives the name of the variable.
		char const* v_name_;
		// The raw pointer on a null terminated string witch gives the name of the weight.
		char const* w_name_;
	}; // struct MyIterator

	if (get.flt_ == nullptr)
		throw msl::Exception{ error_as_interest_variable(f_name, get.name_()) };
	if (weight.flt_ == nullptr)
		throw msl::Exception{ error_as_weight_variable(f_name, get.name_()) };

	// Be sure to call the ``cbegin'' and ``cend'' function members of the base class.
	return std::reduce(
	  std::execution::par_unseq,
	  MyIterator{ this->cbegin(), get.flt_, weight.flt_, f_name, get.name_(), weight.name_() },
	  MyIterator{ this->cend(), get.flt_, weight.flt_, f_name, get.name_(), weight.name_() },
	  double{});
}
